package	jp.co.sds_corp.util.daemon.core;

import java.io.Serializable;

import jp.co.sds_corp.util.common.NoSuchCodeException;
import jp.co.sds_corp.util.config.IConfigSavable;
import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.config.UnspecifiedPropertyException;

/**
時間間隔を保持するクラスです。
<PRE>
&lt;interval type="sec"&gt;10&lt;/interval&gt;
</PRE>
タイプには、 millisec,sec,min,hour,dateを指定することが可能です。
*/
public class Interval implements IConfigurable, IConfigSavable, Serializable
{
	/** 時間間隔 */
	public enum Type {
		/** 時間間隔：ミリ秒 */
		MILLISEC,
		/** 時間間隔：秒 */
		SEC,
		/** 時間間隔：分 */
		MIN,
		/** 時間間隔：時 */
		HOUR,
		/** 時間間隔：日 */
		DATE;
	}
	Type	type;
	long	value;
	long	milliSec;

	/**
	* コンストラクタ
	*/
	public Interval() {
	}

	/**
	* コンストラクタ
	* @param    milliSec    秒数
	*/
	public Interval( long milliSec ) {
		this.type = Type.MILLISEC;
		this.milliSec = milliSec;
	}

	/**
	* 設定情報から秒数を取得します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		milliSec = -1;
		if ( config == null ) {
			return;
		}

		long lRet = -1;
		value = Long.valueOf( config.get() ).longValue();
		type = getType( config );
		switch ( type ) {
			case MILLISEC: lRet = value; break;
			case SEC: lRet = value * 1000; break;
			case MIN: lRet = value * 1000 * 60; break;
			case HOUR: lRet = value * 1000 * 60 * 60; break;
			case DATE: lRet = value * 1000 * 60 * 60 * 24; break;
		}
		milliSec = lRet;
		if ( milliSec < 0 ) {
			throw( new IllegalArgumentException( "Negative interval." ));
		}
	}

	/**
	* 設定情報から間隔のタイプを取得します。
	* @param	config	設定情報
	* @return	間隔のタイプ
	*/
	protected Type getType( IConfiguration config ) throws Exception {
		String type = null;
		try {
			type = config.getAttributes().get( "type" );
		} catch ( UnspecifiedPropertyException ex ) {
			return Type.MILLISEC;
		}
		Type ret = parseType( type );
		return ret;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param    config  設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		String s = "";
		switch ( parseType( milliSec ) ) {
		case MILLISEC: s = "millisec"; break;
		case SEC: s = "sec"; break;
		case MIN: s = "min"; break;
		case HOUR: s = "hour"; break;
		case DATE: s = "date"; break;
		default: s = "unknown";
		}
		long l = parseUnit( milliSec );
		config.setAttribute( "type", s );
		config.set( String.valueOf( l ));
	}

	/**
	* 設定されている内容を文字列形式で出力します。
	* @return	設定されている内容
	*/
	public String toString() {
		String s = parseType( milliSec ).toString();
		long l = parseUnit( milliSec );
		String ret = getClass().getName() + "[" + l + s + "]";
		return ret;
	}

	/**
	* 設定情報から間隔のタイプを取得します。
	* @param	var		設定情報
	* @return	間隔のタイプ
	*/
	protected static Type parseType( String var ) throws Exception {
		if ( var.equals( "millisec" )) {
			return Type.MILLISEC;
		} else
		if ( var.equals( "sec" )) {
			return Type.SEC;
		} else
		if ( var.equals( "min" )) {
			return Type.MIN;
		} else
		if ( var.equals( "hour" )) {
			return Type.HOUR;
		} else
		if ( var.equals( "date" )) {
			return Type.DATE;
		}
		throw( new NoSuchCodeException( "type [" + var + "] not found" ));
	}

	/**
	* ミリ秒がどの単位かを判定する。
	* @param	l		ミリ秒
	*/
	public static Type parseType( long l ) {
		if (( l % 1000 ) != 0 ) {
			return Type.MILLISEC;
		}
		l = l / 1000;
		if (( l % 60 ) != 0 ) {
			return Type.SEC;
		}
		l = l / 60;
		if (( l % 60 ) != 0 ) {
			return Type.MIN;
		}
		l = l / 60;
		if (( l % 24 ) != 0 ) {
			return Type.HOUR;
		}
		return Type.DATE;
	}

	/**
	* ミリ秒を単位に変換した値を取得する。
	* @param	l		ミリ秒
	*/
	public static long parseUnit( long l ) {
		long ret;
		switch ( parseType( l ) ) {
		case MILLISEC: ret = l; break;
		case SEC: ret = l / 1000; break;
		case MIN: ret = l / 1000 / 60; break;
		case HOUR: ret = l / 1000 / 60 / 60; break;
		case DATE: ret = l / 1000 / 60 / 60 / 24; break;
		default: ret = -1;
		}
		return ret;
	}

	/**
	* 起動間隔(milliSec)を設定します。
	* @param	var		設定するデータ
	*/
	public void setMilliSec( long var ) {
		this.milliSec = var ;
	}
	/**
	* 起動間隔(milliSec)を取得します。
	* @return	milliSecの内容
	*/
	public long getMilliSec() {
		return milliSec;
	}

	/**
	* タイプ(type)を設定します。
	* @param	var		設定するデータ
	*/
	public void setType( Type var ) {
		this.type = var ;
	}
	/**
	* タイプ(type)を取得します。
	* @return	typeの内容
	*/
	public Type getType() {
		return type;
	}

	/**
	* 設定値(value)を設定します。
	* @param	var		設定するデータ
	*/
	public void setValue( long var ) {
		this.value = var ;
	}
	/**
	* 設定値(value)を取得します。
	* @return	valueの内容
	*/
	public long getValue() {
		return value;
	}

}

